import os.path

from cpl_cli.configuration import WorkspaceSettings, ProjectSettings
from cpl_core.application import StartupABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console
from cpl_core.dependency_injection import ServiceProviderABC, ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironment

from post_build.service.dependencies import Dependencies
from post_build.service.remove_config import RemoveConfig


class Startup(StartupABC):

    def __init__(self):
        StartupABC.__init__(self)

    def configure_configuration(self, configuration: ConfigurationABC, environment: ApplicationEnvironment) -> ConfigurationABC:
        configuration.add_json_file(f'appsettings.json', optional=False, output=False)
        environment.set_working_directory(os.path.abspath(os.path.join(environment.working_directory, '../../')))
        configuration.add_json_file(f'cpl-workspace.json', optional=False, output=False)
        ws: WorkspaceSettings = configuration.get_configuration(WorkspaceSettings)
        configuration.add_json_file(ws.projects[ws.default_project], optional=False, output=False)

        return configuration

    def configure_services(self, services: ServiceCollectionABC, environment: ApplicationEnvironment) -> ServiceProviderABC:
        services.add_transient(RemoveConfig)
        services.add_transient(Dependencies)

        return services.build_service_provider()
