import { Component, Input, OnChanges, OnInit, SimpleChanges } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { LangChangeEvent, TranslateService } from '@ngx-translate/core';
import { MenuItem } from 'primeng/api';
import { AuthRoles } from 'src/app/models/auth/auth-roles.enum';
import { AuthService } from 'src/app/services/auth/auth.service';
import { ServerService } from 'src/app/services/data/server.service';
import { ThemeService } from 'src/app/services/theme/theme.service';

@Component({
  selector: 'app-sidebar',
  templateUrl: './sidebar.component.html',
  styleUrls: ['./sidebar.component.scss']
})
export class SidebarComponent implements OnInit {

  isSidebarOpen: boolean = true;
  menuItems!: MenuItem[];

  private serverId!: number;

  constructor(
    private authService: AuthService,
    private translateService: TranslateService,
    private themeService: ThemeService,
    private route: ActivatedRoute,
    private serverService: ServerService
  ) {
    this.themeService.isSidebarOpen$.subscribe(value => {
      this.isSidebarOpen = value;
      this.setMenu();
    });

    this.translateService.onLangChange.subscribe((event: LangChangeEvent) => {
      this.setMenu();
    });

    this.serverService.server$.subscribe(server => {
      if (!server) {
        return;
      }

      this.serverId = server.serverId;
      this.setMenu();
    });
  }

  ngOnInit(): void {
    this.themeService.loadMenu();
  }

  setMenu() {
    this.authService.hasUserPermission(AuthRoles.Admin).then(hasPermission => {
      this.menuItems = [];
      this.menuItems = [
        { label: this.isSidebarOpen ? this.translateService.instant('sidebar.dashboard') : '', icon: 'pi pi-th-large', routerLink: 'dashboard' }
      ];

      if (this.serverId) {
        this.addServerMenu();
      }

      if (hasPermission) {
        this.addAdminMenu();
      }
      this.menuItems = this.menuItems.slice();
    });
  }

  addServerMenu() {
    this.menuItems.push(
      {
        label: this.isSidebarOpen ? this.translateService.instant('sidebar.server') : '', icon: 'pi pi-server', items: [
          { label: this.isSidebarOpen ? this.translateService.instant('sidebar.settings') : '', icon: 'pi pi-cog', routerLink: 'server/settings' },
          { label: this.isSidebarOpen ? this.translateService.instant('sidebar.members') : '', icon: 'pi pi-users', routerLink: 'server/members' },
        ]
      }
    );
  }

  addAdminMenu() {
    this.menuItems.push(
      {
        label: this.isSidebarOpen ? this.translateService.instant('sidebar.administration') : '', icon: 'pi pi-cog', items: [
          { label: this.isSidebarOpen ? this.translateService.instant('sidebar.config') : '', icon: 'pi pi-cog', routerLink: '/admin/settings' },
          { label: this.isSidebarOpen ? this.translateService.instant('sidebar.auth_user_list') : '', icon: 'pi pi-user-edit', routerLink: '/admin/users' },
        ]
      },
    );
  }

}
